package com.huiju.estateagents.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.entity.TaChannel;
import com.huiju.estateagents.entity.TsActivityDaily;
import com.huiju.estateagents.excel.ActivityStatistics.ChannelAddRegistNum;
import com.huiju.estateagents.excel.ActivityStatistics.ChannelVisitNum;
import com.huiju.estateagents.excel.ActivityStatistics.ChannelVisitPersonNum;
import com.huiju.estateagents.excel.ActivityStatistics.VisitPersonNum;
import com.huiju.estateagents.mapper.TaChannelMapper;
import com.huiju.estateagents.service.TaChannelService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.time.LocalDate;

/**
 * <p>
 *   服务实现类
 * </p>
 *
 * @author jobob
 * @since 2019-09-17
 */
@Service
public class TaChannelServiceImpl extends ServiceImpl<TaChannelMapper, TaChannel> implements TaChannelService {

    @Autowired
    private TaChannelMapper taChannelMapper;

    /**
     * 渠道引进客户统计
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @param startDate
     * @param endDate
     * @param channelType
     * @param buildingId
     * @return
     */
    @Override
    public ResponseBean getChannelIntroductionList(Integer pageNum, Integer pageSize, Integer orgId, LocalDate startDate, LocalDate endDate, String channelType, String buildingId, Integer channelId) {
        IPage<TsActivityDaily> pg = new Page<>(pageNum, pageSize);
        IPage<TsActivityDaily> data = taChannelMapper.getChannelIntroductionList(pg, orgId,startDate,endDate,channelType,buildingId, channelId);
        return ResponseBean.success(data);
    }

    /**
     * 获取注册用户列表
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @param channelId
     * @param targetId
     * @param targetType
     * @param buildingId
     * @return
     */
    @Override
    public ResponseBean getNewuserList(Integer pageNum, Integer pageSize, Integer orgId, Integer channelId, String targetId, String targetType, String buildingId) {
        IPage<ChannelAddRegistNum> iPage = new Page<>(pageNum, pageSize);
        IPage<ChannelAddRegistNum> data = taChannelMapper.getChannelNewuserList(iPage, orgId, channelId, targetId,targetType, buildingId);
        return ResponseBean.success(data);
    }

    /**
     * 获取渠道下的访问次数列表
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @param channelId
     * @param targetId
     * @param targetType
     * @param buildingId
     * @return
     */
    @Override
    public ResponseBean getVisitNumList(Integer pageNum, Integer pageSize, Integer orgId, Integer channelId, String targetId, String targetType, String buildingId) {
        IPage<ChannelVisitNum> iPage = new Page<>(pageNum, pageSize);
        IPage<ChannelVisitNum> data = taChannelMapper.getChannelVisitNumList(iPage, orgId, channelId, targetId, targetType, buildingId);
        return ResponseBean.success(data);
    }

    /**
     * 获取渠道下的访问人数列表
     *
     * @param pageNum
     * @param pageSize
     * @param orgId
     * @param channelId
     * @param targetId
     * @param targetType
     * @param buildingId
     * @return
     */
    @Override
    public ResponseBean getPersonNumList(Integer pageNum, Integer pageSize, Integer orgId, Integer channelId, String targetId, String targetType, String buildingId) {
        IPage<ChannelVisitPersonNum> iPage = new Page<>(pageNum, pageSize);
        IPage<ChannelVisitPersonNum> data = taChannelMapper.getChannelPersonNumList(iPage, orgId, channelId, targetId, targetType, buildingId);
        return ResponseBean.success(data);
    }
}
