package com.huiju.estateagents.reportCustomerAop;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.servlet.http.HttpServletRequest;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import com.alibaba.fastjson.JSONObject;
import com.huiju.estateagents.base.BaseController;
import com.huiju.estateagents.base.ResponseBean;
import com.huiju.estateagents.common.CommConstant;
import com.huiju.estateagents.common.StringUtils;
import com.huiju.estateagents.entity.TaBuilding;
import com.huiju.estateagents.entity.TaCustomerFrom;
import com.huiju.estateagents.entity.TaPerson;
import com.huiju.estateagents.mapper.TaBuildingMapper;
import com.huiju.estateagents.mapper.TaPersonMapper;
import com.huiju.estateagents.service.ITaRecommendCustomerService;

import io.swagger.models.auth.In;

@Aspect
@Component
public class ReportCustomerAspect extends BaseController {

    @Autowired
    private ITaRecommendCustomerService iTaRecommendCustomerService;

    @Autowired
    TaPersonMapper taPersonMapper;

    @Autowired
    private TaBuildingMapper taBuildingMapper;


    /** 以自定义 @ReportCustomer 注解为切点 */
    @Pointcut("@annotation(com.huiju.estateagents.reportCustomerAop.ReportCustomer)")
    public void reportCustomer() {

    }

    @Before("reportCustomer()")
    public ResponseBean doAround(JoinPoint proceedingJoinPoint) throws Throwable {
        ResponseBean responseBean = new ResponseBean();
        try{
            ReportCustomer reportCustomer = ((MethodSignature)proceedingJoinPoint.getSignature()).getMethod().getAnnotation(ReportCustomer.class);
            //获取请求参数
            Object[] args = proceedingJoinPoint.getArgs();
            HttpServletRequest request = (HttpServletRequest)args[1];

            if (request == null){
                responseBean.addError("未授权，不进行操作");
                return responseBean;
            }

            String params = request.getHeader("reportInfo");
            JSONObject paramObject = JSONObject.parseObject(params);

            List<TaPerson> persons = getPersonsByOpenId(getOpenId(request));
            TaPerson taPerson = persons.get(0);
            Integer orgId = getOrgId(request);

            // 楼盘ID
            String buildingId = paramObject.getString("buildingId");
            // 场景ID
            String sceneId = paramObject.getString("sceneId");
            // 访问目标
            String targetType = reportCustomer.type();
            // 目标ID
            String targetId = paramObject.getString("targetId");
            // 目标名称
            String targetName = paramObject.getString("targetName");
            // 推广用户
            String promoter = paramObject.getString("promoter");

            TaCustomerFrom  taCustomerFrom = assembleVO(taPerson, targetType, targetId, targetName, orgId, sceneId);
            // 推广人
            TaPerson recPerson = StringUtils.isEmpty(promoter) ? null : taPersonMapper.getById(promoter);
            // 是否置业顾问
            boolean isConsultant = null != recPerson && CommConstant.PERSON_REALTY_CONSULTANT.equals(recPerson.getPersonType());
            // 推荐楼盘
            TaBuilding building = StringUtils.isEmpty(buildingId) ? null : taBuildingMapper.selectById(buildingId);

            iTaRecommendCustomerService.newCustomer(orgId, taPerson, recPerson, isConsultant ? recPerson : null, building, taCustomerFrom);
            responseBean.addSuccess("绑定成功");

        }catch (Exception e){
            responseBean.addError("失败");
        }
        return responseBean;
    }

    public TaCustomerFrom assembleVO(TaPerson person, String targetType, String targetId, String targetName, Integer orgId, String sceneId){
        // 客户来源
        TaCustomerFrom customerFrom = new TaCustomerFrom();
        customerFrom.setPersonId(person.getPersonId());
        customerFrom.setPersonName(StringUtils.ifNull(person.getName(), person.getNickname()));
        customerFrom.setTargetType(targetType);
        customerFrom.setTargetId(targetId);
        customerFrom.setTargetName(targetName);
        customerFrom.setOrgId(orgId);
        customerFrom.setSceneId(sceneId);
        customerFrom.setCreateDate(LocalDateTime.now());

        return customerFrom;
    }

    public List<TaPerson> getPersonsByOpenId(String openId){
        return taPersonMapper.getPersonByOpenId(openId);
    }
}
